SET SERVEROUTPUT ON SIZE 1000000

DECLARE

  v_major INTEGER := 0;
  v_minor INTEGER := 0;
  v_patch INTEGER := 0;
  v_upgrade_state VARCHAR2(24);

  v_req_major     CONSTANT INTEGER  := 3;
  v_req_minor     CONSTANT INTEGER  := 4;
  v_req_patch_min CONSTANT INTEGER  := 0;
  v_req_patch_max CONSTANT INTEGER  := 0;

  v_next_major CONSTANT INTEGER := 3;
  v_next_minor CONSTANT INTEGER := 4;
  v_next_patch CONSTANT INTEGER := 1;

  -- verify if the 'Online Index Build' option is supported (disabled in Oracle XE)
  online_index_build V$OPTION.VALUE%TYPE := NULL;

-- Main
BEGIN

  DBMS_OUTPUT.PUT_LINE('Applying Patch...');
  -- Get schema version
  SELECT MAJOR,MINOR,PATCH INTO v_major,v_minor,v_patch FROM T_SCHEMA_VERS WHERE ROWNUM=1;
  -- check version
  IF v_major = v_req_major AND  v_minor = v_req_minor AND (v_patch BETWEEN  v_req_patch_min AND v_req_patch_max) THEN
      -- apply patch

      -- Add a schema state column that can be used to check if a previous upgrade attempt failed
      DBMS_OUTPUT.PUT_LINE('Upgrading T_SCHEMA_VERS...');
      -- set schema state
      EXECUTE IMMEDIATE('UPDATE T_SCHEMA_VERS SET STATE=''Upgrading''');
      DBMS_OUTPUT.PUT_LINE('Upgrading schema ' || v_major || '.' || v_minor || '.' || v_patch || ' to ' || v_next_major || '.' || v_next_minor || '.' || v_next_patch || ' ...');

      --
      -- Add the Checksum_method to t_job

      EXECUTE IMMEDIATE ('ALTER TABLE T_JOB ADD CHECKSUM_METHOD  CHAR(1) DEFAULT  NULL');
      EXECUTE IMMEDIATE ('ALTER TABLE T_JOB_HISTORY ADD CHECKSUM_METHOD  CHAR(1) DEFAULT  NULL');

      DBMS_OUTPUT.PUT_LINE('Updating schema version...');
      UPDATE T_SCHEMA_VERS SET MAJOR=v_next_major, MINOR=v_next_minor, PATCH=v_next_patch; 
      EXECUTE IMMEDIATE ('UPDATE T_SCHEMA_VERS SET STATE=NULL'); 
      DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch applied. New version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
 ELSIF v_major = v_next_major AND  v_minor = v_next_minor AND v_patch = v_next_patch THEN
     DBMS_OUTPUT.PUT_LINE('SUCCESS: Patch already applied. Version is ' || TO_CHAR(v_next_major) || '.' || TO_CHAR(v_next_minor) || '.' || TO_CHAR(v_next_patch) );
  ELSE
     DBMS_OUTPUT.PUT_LINE('ERROR: Cannot apply patch. Version ' || TO_CHAR(v_major) || '.' || TO_CHAR(v_minor) || '.' || TO_CHAR(v_patch) || ' does not match the required one (' || TO_CHAR(v_req_major) || '.' || TO_CHAR(v_req_minor) || '.' || TO_CHAR(v_req_patch_min) || '-' ||  TO_CHAR(v_req_patch_max) || ')');
  END IF;
END;
/
exit;
